@@ -42,7 +42,6 @@
 #include <eel/eel-labeled-image.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-vfs-extensions.h>
 #include <eel/eel-xml-extensions.h>
 #include <librsvg/rsvg.h>
 #include <libxml/parser.h>
@@ -67,6 +66,8 @@
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkviewport.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
+#include <gio/gio.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-color-picker.h>
@@ -74,12 +75,10 @@
 #include <libgnomeui/gnome-help.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomeui/gnome-uidefs.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <libnautilus-private/nautilus-customization-data.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-emblem-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-metadata.h>
@@ -690,7 +689,7 @@ nautilus_property_browser_drag_data_get (GtkWidget *widget,
 			g_free (user_directory);
 		}
 
-		image_file_uri = gnome_vfs_get_uri_from_local_path (image_file_name);
+		image_file_uri = g_filename_to_uri (image_file_name, NULL, NULL);
 		gtk_selection_data_set (selection_data, selection_data->target, 8, image_file_uri, strlen (image_file_uri));
 		g_free (image_file_name);
 		g_free (image_file_uri);
@@ -714,22 +713,26 @@ nautilus_property_browser_drag_end (GtkWidget *widget, GdkDragContext *context)
 
 /* utility routine to check if the passed-in uri is an image file */
 static gboolean
-ensure_uri_is_image (const char *uri)
-{	
-	gboolean is_image;
-	GnomeVFSResult result;
-	GnomeVFSFileInfo *file_info;
+ensure_file_is_image (GFile *file)
+{
+	GFileInfo *info;
+	const char *mime_type;
 
-	file_info = gnome_vfs_file_info_new ();
-	result = gnome_vfs_get_file_info
-		(uri, file_info,
-		 GNOME_VFS_FILE_INFO_GET_MIME_TYPE
-		 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-        is_image = eel_istr_has_prefix (file_info->mime_type, "image/")
-		&& eel_strcasecmp (file_info->mime_type, "image/svg") != 0
-		&& eel_strcasecmp (file_info->mime_type, "image/svg+xml") != 0;
-	gnome_vfs_file_info_unref (file_info);
-	return is_image;
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, 0, NULL, NULL);
+	if (info == NULL) {
+		return FALSE;
+	}
+
+	mime_type = g_file_info_get_content_type (info);
+	if (mime_type == NULL) {
+		return FALSE;
+	}
+
+	g_object_unref (info);
+
+	return  g_content_type_is_a (mime_type, "image/*") &&
+		!g_content_type_equals (mime_type, "image/svg") &&
+		!g_content_type_equals (mime_type, "image/svg+xml");
 }
 
 /* create the appropriate pixbuf for the passed in file */
@@ -741,6 +744,7 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 	char *image_file_name;
 	char *icon_name;
 	gboolean is_reset;
+	NautilusIconInfo *info;
 
 	if (property_browser->details->category_type == NAUTILUS_PROPERTY_EMBLEM) {
 		if (strcmp (file_name, "erase") == 0) {
@@ -753,9 +757,9 @@ make_drag_image (NautilusPropertyBrowser *property_browser, const char* file_nam
 			g_free (image_file_name);
 		} else {
 			icon_name = nautilus_emblem_get_icon_name_from_keyword (file_name);
-			pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL,
-									     NAUTILUS_ICON_SIZE_STANDARD, TRUE,
-									     NULL);
+			info = nautilus_icon_info_lookup_from_name (file_name, NAUTILUS_ICON_SIZE_STANDARD);
+			pixbuf = nautilus_icon_info_get_pixbuf_at_size (info, NAUTILUS_ICON_SIZE_STANDARD);
+			g_object_unref (info);
 			g_free (icon_name);
 		}
 		return pixbuf;
@@ -955,29 +959,27 @@ remove_color (NautilusPropertyBrowser *property_browser, const char* color_name)
 static void
 remove_pattern(NautilusPropertyBrowser *property_browser, const char* pattern_name)
 {
-	char *pattern_path, *pattern_uri;
+	char *pattern_path;
 	char *user_directory;
 
 	user_directory = nautilus_get_user_directory ();
 
 	/* build the pathname of the pattern */
-	pattern_path = g_strdup_printf ("%s/patterns/%s",
-					   user_directory,
-					   pattern_name);
-	pattern_uri = gnome_vfs_get_uri_from_local_path (pattern_path);
-	g_free (pattern_path);
-
+	pattern_path = g_build_filename (user_directory,
+					 "patterns",
+					 pattern_name,
+					 NULL);
 	g_free (user_directory);	
 
 	/* delete the pattern from the pattern directory */
-	if (gnome_vfs_unlink (pattern_uri) != GNOME_VFS_OK) {
+	if (g_unlink (pattern_path) != 0) {
 		char *message = g_strdup_printf (_("Sorry, but pattern %s couldn't be deleted."), pattern_name);
 		char *detail = _("Check that you have permission to delete the pattern.");
 		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
 	}
 	
-	g_free (pattern_uri);
+	g_free (pattern_path);
 }
 
 /* remove the emblem matching the passed in name */
@@ -986,30 +988,28 @@ static void
 remove_emblem (NautilusPropertyBrowser *property_browser, const char* emblem_name)
 {
 	/* build the pathname of the emblem */
-	char *emblem_path, *emblem_uri;
+	char *emblem_path;
 	char *user_directory;
 
 	user_directory = nautilus_get_user_directory ();
 
-	emblem_path = g_strdup_printf ("%s/emblems/%s",
-				       user_directory,
-				       emblem_name);
-	emblem_uri = gnome_vfs_get_uri_from_local_path (emblem_path);
-	g_free (emblem_path);
+	emblem_path = g_build_filename (user_directory,
+					"emblems",
+					emblem_name,
+					NULL);
 
 	g_free (user_directory);
 
 	/* delete the emblem from the emblem directory */
-	if (gnome_vfs_unlink (emblem_uri) != GNOME_VFS_OK) {
+	if (g_unlink (emblem_path) != 0) {
 		char *message = g_strdup_printf (_("Sorry, but emblem %s couldn't be deleted."), emblem_name);
 		char *detail = _("Check that you have permission to delete the emblem.");
 		eel_show_error_dialog (message, detail, GTK_WINDOW (property_browser));
 		g_free (message);
-	}
-	else {
+	} else {
 		emit_emblems_changed_signal ();
 	}
-	g_free (emblem_uri);
+	g_free (emblem_path);
 }
 
 /* handle removing the passed in element */
@@ -1175,8 +1175,7 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 	char *directory_path, *source_file_name, *destination_name;
 	char *path_uri, *basename;
 	char *user_directory;	
-	char *directory_uri;
-	GnomeVFSResult result;
+	GFile *src, *dest;
 	
 	NautilusPropertyBrowser *property_browser = NAUTILUS_PROPERTY_BROWSER (data);
 
@@ -1197,10 +1196,10 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 	}
 	
 	/* fetch the mime type and make sure that the file is an image */
-	path_uri = gnome_vfs_get_uri_from_local_path (path_name);	
+	path_uri = g_filename_to_uri (path_name, NULL, NULL);	
 
 	/* don't allow the user to change the reset image */
-	basename = eel_uri_get_basename (path_uri);
+	basename = g_path_get_basename (path_name);
 	if (basename && eel_strcmp (basename, RESET_IMAGE_NAME) == 0) {
 		eel_show_error_dialog (_("Sorry, but you can't replace the reset image."), 
 		                       _("Reset is a special image that cannot be deleted."), 
@@ -1222,24 +1221,24 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 	destination_name = g_build_filename (directory_path, source_file_name + 1, NULL);
 
 	/* make the directory if it doesn't exist */
-	if (!g_file_test(directory_path, G_FILE_TEST_EXISTS)) {
-		directory_uri = gnome_vfs_get_uri_from_local_path (directory_path);
-		gnome_vfs_make_directory (directory_uri,
-						 GNOME_VFS_PERM_USER_ALL
-						 | GNOME_VFS_PERM_GROUP_ALL
-						 | GNOME_VFS_PERM_OTHER_READ);
-		g_free (directory_uri);
+	if (!g_file_test (directory_path, G_FILE_TEST_EXISTS)) {
+		g_mkdir_with_parents (directory_path, 0775);
 	}
 		
 	g_free (directory_path);
-		
-	result = eel_copy_uri_simple (path_name, destination_name);		
-	if (result != GNOME_VFS_OK) {
+
+	src = g_file_new_for_path (path_name);
+	dest = g_file_new_for_path (destination_name);
+	if (!g_file_copy (src, dest,
+			  0,
+			  NULL, NULL, NULL, NULL)) {
 		char *message = g_strdup_printf (_("Sorry, but the pattern %s couldn't be installed."), path_name);
 		eel_show_error_dialog (message, NULL, GTK_WINDOW (property_browser));
 		g_free (message);
 	}
-		
+	g_object_unref (src);
+	g_object_unref (dest);
+	
 	g_free (destination_name);
 	
 	/* update the property browser's contents to show the new one */
@@ -1320,17 +1319,17 @@ add_color_to_file (NautilusPropertyBrowser *property_browser, const char *color_
 static void
 add_color_to_browser (GtkWidget *widget, gint which_button, gpointer *data)
 {
-	char *color_spec;
+	char color_spec[8];
 	const char *color_name;
 	char *stripped_color_name;
-	
+
 	gdouble color[4];
 	NautilusPropertyBrowser *property_browser = NAUTILUS_PROPERTY_BROWSER (data);
 
 	if (which_button == GTK_RESPONSE_OK) {
 		gnome_color_picker_get_d (GNOME_COLOR_PICKER (property_browser->details->color_picker), &color[0], &color[1], &color[2], &color[3]);		
-		color_spec = g_strdup_printf
-			("#%02X%02X%02X",
+		g_snprintf (color_spec, sizeof (color_spec),
+			 "#%02X%02X%02X",
 			 (guint) (color[0] * 255.0 + 0.5),
 			 (guint) (color[1] * 255.0 + 0.5),
 			 (guint) (color[2] * 255.0 + 0.5));
@@ -1347,8 +1346,7 @@ add_color_to_browser (GtkWidget *widget, gint which_button, gpointer *data)
 			nautilus_property_browser_update_contents(property_browser);
 		}
 		g_free (stripped_color_name);
-		g_free(color_spec);	
-	} 
+	}
 	
 	gtk_widget_destroy(property_browser->details->colors_dialog);
 	property_browser->details->colors_dialog = NULL;
@@ -1414,17 +1412,18 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 {
 	const char *new_keyword;
 	char *stripped_keyword;
-	char *emblem_path, *emblem_uri;
+	char *emblem_path;
+	GFile *emblem_file;
 	GdkPixbuf *pixbuf;
-	 
+
 	if (which_button == GTK_RESPONSE_OK) {
 
 		/* update the image path from the file entry */
 		if (property_browser->details->file_entry) {
 			emblem_path = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (property_browser->details->file_entry));
 			if (emblem_path) {
-				emblem_uri = gnome_vfs_get_uri_from_local_path (emblem_path);
-				if (ensure_uri_is_image (emblem_uri)) {
+				emblem_file = g_file_new_for_path (emblem_path);
+				if (ensure_file_is_image (emblem_file)) {
 					g_free (property_browser->details->image_path);
 					property_browser->details->image_path = emblem_path;				
 				} else {
@@ -1433,15 +1432,16 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 					eel_show_error_dialog (_("The file is not an image."), message, GTK_WINDOW (property_browser));
 					g_free (message);
 					g_free (emblem_path);
+					g_object_unref (emblem_file);
 					return;
 				}
-				g_free (emblem_uri);
+				g_object_unref (emblem_file);
 			}
 		}
-		
-		emblem_uri = gnome_vfs_get_uri_from_local_path (property_browser->details->image_path);
-		pixbuf = nautilus_emblem_load_pixbuf_for_emblem (emblem_uri);
-		g_free (emblem_uri);
+
+		emblem_file = g_file_new_for_path (property_browser->details->image_path);
+		pixbuf = nautilus_emblem_load_pixbuf_for_emblem (emblem_file);
+		g_object_unref (emblem_file);
 
 		if (pixbuf == NULL) {
 			char *message = g_strdup_printf
@@ -1449,7 +1449,7 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 			eel_show_error_dialog (_("The file is not an image."), message, GTK_WINDOW (property_browser));
 			g_free (message);
 		}
-		
+
 		new_keyword = gtk_entry_get_text(GTK_ENTRY(property_browser->details->keyword));		
 		if (new_keyword == NULL) {
 			stripped_keyword = NULL;
@@ -1707,6 +1707,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 	GtkWidget *blank;
 	guint num_images;
 	char *path;
+	NautilusIconInfo *info;
 
 	g_return_if_fail (NAUTILUS_IS_PROPERTY_BROWSER (property_browser));
 	g_return_if_fail (EEL_IS_IMAGE_TABLE (property_browser->details->content_table));
@@ -1717,7 +1718,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 		eel_g_list_free_deep (property_browser->details->keywords);	
 		property_browser->details->keywords = NULL;
 
-		icons = nautilus_emblem_list_availible ();
+		icons = nautilus_emblem_list_available ();
 
 		l = icons;
 		while (l != NULL) {
@@ -1728,9 +1729,11 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 				continue;
 			}
 			object_name = nautilus_emblem_get_keyword_from_icon_name (icon_name);
-			object_pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_name, NULL,
-										    NAUTILUS_ICON_SIZE_SMALL, TRUE,
-										    &object_label);
+			info = nautilus_icon_info_lookup_from_name (object_name, NAUTILUS_ICON_SIZE_STANDARD);
+			object_pixbuf = nautilus_icon_info_get_pixbuf_at_size (info, NAUTILUS_ICON_SIZE_STANDARD);
+			object_label = g_strdup (nautilus_icon_info_get_display_name (info));
+			g_object_unref (info);
+			
 			if (object_label == NULL) {
 				object_label = g_strdup (object_name);
 			}
@@ -1760,7 +1763,6 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 	} else {
 		customization_data = nautilus_customization_data_new (property_browser->details->category,
 								      !property_browser->details->remove_mode,
-								      FALSE,
 								      MAX_ICON_WIDTH,
 								      MAX_ICON_HEIGHT);
 		if (customization_data == NULL) {
@@ -1771,7 +1773,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 		while (nautilus_customization_data_get_next_element_for_display (customization_data,
 										 &object_name,
 										 &object_pixbuf,
-										 &object_label) == GNOME_VFS_OK) {
+										 &object_label)) {
 			
 			property_image = labeled_image_new (object_label, object_pixbuf, object_name, PANGO_SCALE_LARGE);
 			
@@ -1780,7 +1782,7 @@ make_properties_from_directories (NautilusPropertyBrowser *property_browser)
 			
 			/* Keep track of ERASE objects to place them prominently later */
 			if (property_browser->details->category_type == NAUTILUS_PROPERTY_PATTERN
-			    && eel_str_is_equal (object_name, RESET_IMAGE_NAME)) {
+			    && !eel_strcmp (object_name, RESET_IMAGE_NAME)) {
 				g_assert (reset_object == NULL);
 				reset_object = property_image;
 			}
