@@ -968,11 +968,14 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 		data->cancellable = g_cancellable_new ();
 		data->window = window;
 		window->details->mount_cancellable = data->cancellable;
-		g_file_mount_enclosing_volume (location, mount_op, window->details->mount_cancellable,
+		nautilus_inhibit_autorun_for_file (location);
+		g_file_mount_enclosing_volume (location, 0, mount_op, window->details->mount_cancellable,
 					       mount_not_mounted_callback, data);
 		g_object_unref (location);
 		g_object_unref (mount_op);
-		
+
+		nautilus_file_unref (file);
+
 		return;
 	}
 	
@@ -1333,6 +1336,48 @@ nautilus_window_show_trash_bar (NautilusWindow *window)
 	nautilus_window_add_extra_location_widget (window, bar);
 }
 
+typedef struct {
+	NautilusWindow *window;
+	GCancellable *cancellable;
+} FindMountData;
+
+static void
+found_mount_cb (GObject *source_object,
+		GAsyncResult *res,
+		gpointer user_data)
+{
+	FindMountData *data = user_data;
+	GMount *mount;
+	NautilusWindow *window;	
+	char **x_content_types;
+
+	if (g_cancellable_is_cancelled (data->cancellable)) {
+		goto out;
+	}
+	
+	window = data->window;
+	
+	mount = g_file_find_enclosing_mount_finish (G_FILE (source_object),
+						    res,
+						    NULL);
+	if (mount != NULL) {
+		x_content_types = nautilus_autorun_get_x_content_types_for_mount (mount, FALSE);
+		if (x_content_types != NULL && x_content_types[0] != NULL) {
+			nautilus_window_show_x_content_bar (window, mount, x_content_types);
+			update_extra_location_widgets_visibility (window);
+		}
+		g_strfreev (x_content_types);
+		
+		g_object_unref (mount);
+	}
+	
+	window->details->find_mount_cancellable = NULL;
+
+ out:
+	g_object_unref (data->cancellable);
+	g_free (data);
+}
+
 /* Handle the changes for the NautilusWindow itself. */
 static void
 update_for_new_location (NautilusWindow *window)
@@ -1342,8 +1387,7 @@ update_for_new_location (NautilusWindow *window)
 	NautilusDirectory *directory;
 	gboolean location_really_changed;
 	char *uri;
-	char **x_content_types;
-	GMount *mount;
+	FindMountData *data;
         
         new_location = window->details->pending_location;
         window->details->pending_location = NULL;
@@ -1402,15 +1446,22 @@ update_for_new_location (NautilusWindow *window)
 			nautilus_window_show_trash_bar (window);
 		}
 
-		mount = nautilus_file_get_mount (file);
-		if (mount != NULL) {
-			x_content_types = nautilus_autorun_get_x_content_types_for_mount (mount, FALSE);
-			if (x_content_types != NULL && x_content_types[0] != NULL) {
-				nautilus_window_show_x_content_bar (window, mount, x_content_types);
-			}
-			g_strfreev (x_content_types);
-			g_object_unref (mount);
+		/* need the mount to determine if we should put up the x-content cluebar */
+		if (window->details->find_mount_cancellable != NULL) {
+			g_cancellable_cancel (window->details->find_mount_cancellable);
+			window->details->find_mount_cancellable = NULL;
 		}
+		
+		data = g_new (FindMountData, 1);
+		data->window = window;
+		data->cancellable = g_cancellable_new ();
+		
+		window->details->find_mount_cancellable = data->cancellable;
+		g_file_find_enclosing_mount_async (window->details->location, 
+						   G_PRIORITY_DEFAULT, 
+						   data->cancellable,
+						   found_mount_cb,
+						   data);
 
 		nautilus_directory_unref (directory);
 
@@ -1915,7 +1966,6 @@ nautilus_window_reload (NautilusWindow *window)
 {
 	GFile *location;
         char *current_pos;
-	GList *selection_uris, *l;
 	GList *selection;
 	
         g_return_if_fail (NAUTILUS_IS_WINDOW (window));
@@ -1932,13 +1982,7 @@ nautilus_window_reload (NautilusWindow *window)
 	selection = NULL;
 	if (window->content_view != NULL) {
 		current_pos = nautilus_view_get_first_visible_file (window->content_view);
-		selection_uris = nautilus_view_get_selection (window->content_view);
-		selection = NULL;
-		for (l = selection_uris; l != NULL; l = l->next) {
-			selection = g_list_prepend (selection, g_file_new_for_uri (l->data));
-		}
-		eel_g_list_free_deep (selection_uris);
-		selection = g_list_reverse (selection);
+		selection = nautilus_view_get_selection (window->content_view);
 	}
 	begin_location_change
 		(window, location, selection,
