@@ -33,14 +33,14 @@
 void                    nautilus_window_manage_views_destroy          (NautilusWindow           *window);
 void                    nautilus_window_manage_views_finalize         (NautilusWindow           *window);
 void                    nautilus_window_open_location                 (NautilusWindow           *window,
-                                                                       const char               *location,
+                                                                       GFile                    *location,
                                                                        gboolean                  close_behind);
 void                    nautilus_window_open_location_with_selection  (NautilusWindow           *window,
-                                                                       const char               *location,
+                                                                       GFile                    *location,
                                                                        GList                    *selection,
                                                                        gboolean                  close_behind);
 void                    nautilus_window_open_location_full            (NautilusWindow           *window,
-                                                                       const char               *location,
+                                                                       GFile                    *location,
                                                                        NautilusWindowOpenMode    mode,
                                                                        NautilusWindowOpenFlags   flags,
                                                                        GList                    *new_selection);
@@ -51,6 +51,8 @@ gboolean                nautilus_window_content_view_matches_iid      (NautilusW
                                                                        const char               *iid);
 const char             *nautilus_window_get_content_view_id           (NautilusWindow           *window);
 char                   *nautilus_window_get_view_label                (NautilusWindow           *window);
+char                   *nautilus_window_get_view_error_label          (NautilusWindow           *window);
+char                   *nautilus_window_get_view_startup_error_label  (NautilusWindow           *window);
 void                    nautilus_navigation_window_set_sidebar_panels (NautilusNavigationWindow *window,
                                                                        GList                    *view_identifier_list);
 
